/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla addressbook.
 *
 * The Initial Developer of the Original Code is
 * Pawel Chmielowski <prefiks@o2.pl>
 *
 * Portions created by the Initial Developer are Copyright (C) 2002
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Pawel Chmielowski <prefiks@o2.pl>
 * Dominik Tomaszuk <ddooss@wp.pl>
 *
 * ***** END LICENSE BLOCK ***** */

function BarMLHandler()
{
    var content;
    var _this = this;

    this.contentGenerator = new BarMLContentGenerator();

    this.linkDocument = document.implementation.
        createDocument("", "barml-data", null);
    this.linkDocument.addEventListener("load", function (event) { _this.onBarDataLoaded(event) }, false);

    document.addEventListener("unload", this.clean, false);

    content = document.getElementById("appcontent");
    content.addEventListener("DOMLinkAdded", function (event) { _this.updateBar(event) }, false);
    content.addEventListener("unload", function (event) { _this.doHideMyBar(event) }, true);
    content.addEventListener("select", function (event) { _this.onSelect(event) }, false);

    this.mozBarEnabled = true;
    this.myBarVisible = false;
}

BarMLHandler.prototype =
{
    updateBar: function(event)
    {
        var el = event.originalTarget;
        var el2 = el.parentNode;
        var ios, uri;

        if (!("myBarEnabled" in this))
            this.myBarEnabled = document.getElementById("barml_menu_show").
                getAttribute("checked") == "true";

        if (!this.myBarEnabled)
            return;

        if (!el || !el.hasAttribute("rel") || 
                el.getAttribute("rel").toUpperCase() != "BARML" ||
                !el.hasAttribute("href"))
            return;

        while (el2 && !(el2 instanceof Components.interfaces.nsIDOMElement))
            el2 = el2.parentNode;
        if (!el2 || !(el2 instanceof Components.interfaces.nsIDOMHTMLHeadElement))
            return;

        while (el2.previousSibling &&
                !(el2 instanceof Components.interfaces.nsIDOMElement))
            el2 = el2.previousSibling;
        if (el2.prevoiusSibling)
            return;

        el2 = el2.parentNode;
        while (el2 && !(el2 instanceof Components.interfaces.nsIDOMElement))
            el2 = el2.parentNode;
        if (!el2 || !(el2 instanceof Components.interfaces.nsIDOMHTMLHtmlElement))
            return;

        ios = Components.classes["@mozilla.org/network/io-service;1"].
            getService(Components.interfaces.nsIIOService);

        uri = ios.newURI(window.content.location.href, null, null);

        if (barMLHandler.securityCheck(uri.spec))
            barMLHandler.linkDocument.load(uri.resolve(el.getAttribute("href")));
    },

    updateBarSpec: function()
    {
        var el = window.content;
        var ios, uri;

        if (!("myBarEnabled" in this))
            this.myBarEnabled = document.getElementById("barml_menu_show").
                getAttribute("checked") == "true";

        if (!this.myBarEnabled || !el)
            return;

        el = el.document.firstChild;

        while (el && !(el instanceof Components.interfaces.nsIDOMElement))
            el = el.nextSibling;

        if (!el || !(el instanceof Components.interfaces.nsIDOMHTMLHtmlElement))
            return;

        el = el.firstChild;
        while (el && !(el instanceof Components.interfaces.nsIDOMElement))
            el = el.nextSibling;

        if (!el || !(el instanceof Components.interfaces.nsIDOMHTMLHeadElement))
            return;

        el = el.firstChild;
        while (el && !(el instanceof Components.interfaces.nsIDOMHTMLLinkElement && 
                el.hasAttribute("rel") && el.getAttribute("rel").toUpperCase() == "BARML"))
            el = el.nextSibling;

        if (!el || !el.hasAttribute("href"))
            return;

        ios = Components.classes["@mozilla.org/network/io-service;1"].
            getService(Components.interfaces.nsIIOService);

        uri = ios.newURI(window.content.location.href, null, null);

        if (barMLHandler.securityCheck(uri.spec))
            barMLHandler.linkDocument.load(uri.resolve(el.getAttribute("href")));
    },

    onSelect: function(event)
    {
        if (event.originalTarget.localName != "tabs")
            return;
        bar = document.getElementById("barml-toolbar");
        if (!bar)
            return;
        this.doHideMyBar();
        barMLHandler.updateBarSpec();
    },

    onBarDataLoaded: function(event)
    {
        var fragment, bar;
        var doc, el, el2;

        bar = document.getElementById("barml-toolbar");
        if (!bar)
            return;

        while (bar.firstChild)
            bar.removeChild(bar.firstChild);

        el = barMLHandler.appendTransformed(barMLHandler.linkDocument, bar);

        if (!el)
            return;

        this.doShowMyBar();
    },

    appendTransformed: function(dom, root)
    {
        ios = Components.classes["@mozilla.org/network/io-service;1"].
            getService(Components.interfaces.nsIIOService);

        this.contentGenerator.baseURI = 
            ios.newURI(window.content.location.href, null, null);

        return this.contentGenerator.build(dom, root);
    },

    clean: function()
    {
        content = document.getElementById("appcontent");
        content.removeEventListener("DOMLinkAdded", this.updateBar, false);
        content.removeEventListener("unload", this.hideBar, true);
        content.removeEventListener("select", this.onSelect, false);
        document.removeEventListener("unload", this.clean, true);
    },

    doHideMyBar: function()
    {
        var bar = document.getElementById("barml-toolbar");
        bar.setAttribute("hidden", "true");
        this.myBarVisible = false;

        if (!this.mozBarEnabled)
            this.doShowMozBar();
    },

    doShowMyBar: function()
    {
        var bar = document.getElementById("barml-toolbar");
        bar.setAttribute("hidden", "false");
        this.myBarVisible = true;

        if (!this.mozBarEnabled)
            this.doHideMozBar();
    },

    doHideMozBar: function()
    {
/*        if (linkToolbarUI.isLinkToolbarEnabled())
            linkToolbarUI.toggleLinkToolbar(
                document.getElementById("linktoolbar"));*/
    },

    doShowMozBar: function()
    {
/*        if (!linkToolbarUI.isLinkToolbarEnabled())
            linkToolbarUI.toggleLinkToolbar(
                document.getElementById("linktoolbar"));*/
    },

    openLink: function(uri, target)
    {
        try {
            if (barMLHandler.securityCheck(uri)) {
                if (target == "_self")
                    window.content.location.href = uri;
                else
                    window.open(uri, target);
            }
        } catch (ex) {
        }
    },

    securityCheck: function(uri)
    {
        var ssm = Components.classes["@mozilla.org/scriptsecuritymanager;1"].getService().
            QueryInterface(Components.interfaces.nsIScriptSecurityManager);

        try {
            ssm.checkLoadURIStr(window.content.location.href, uri, 0);
        } catch (ex) {
            return false;
        }
        return true;
    },

    menuCommand: function(type)
    {
        switch (type) {
        case "hide":
            this.myBarEnabled = false;
            document.getElementById("barml_menu_show").
                setAttribute("checked", "false");
            this.doHideMyBar();
            break;
        case "show":
            this.myBarEnabled = true;
            document.getElementById("barml_menu_hide").
                setAttribute("checked", "false");
            this.updateBarSpec();
            break;
        case "hide_mozlinkbar":
            this.mozBarEnabled = !this.mozBarEnabled;

            if (this.mozBarEnabled) {
                if (this.myBarVisible)
                    this.doHideMozBar();
            } else
                this.doShowMozBar();
            break;
        }
    },

    initMenu: function()
    {
/*        if (!("myBarEnabled" in this))
            this.myBarEnabled = document.getElementById("barml_menu_show").
                getAttribute("value") != "false";

        document.getElementById("barml_menu_show").
            setAttribute("value", this.myBarEnabled ? "true" : "false");
        document.getElementById("barml_menu_hide").value = !this.myBarEnabled
            setAttribute("value",  ? "false" : "true");
        document.getElementById("barml_menu_hide_mozlinkbar").
            setAttribute("checked", this.mozBarEnabled ? "false" : "true");*/
    }
}

function BarMLContentGenerator()
{
    this.ios = Components.classes["@mozilla.org/network/io-service;1"].
        getService(Components.interfaces.nsIIOService);
}

BarMLContentGenerator.prototype =
{
    xulns: "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul",

    setbaseURI: function(uri)
    {
        this.baseURI = uri;
    },

    build: function(dom, root)
    {
        var el = this.firstChildElement(dom);
        var oldbaseURI, title;
        var baseURI = this.baseURI;

        if (!baseURI)
            return null

        if (!el || el.localName != "bar")
            return null;

        title = el.getAttribute("title");

        if (el.hasAttribute("xml:base"))
            baseURI = ios.newURI(el.getAttribute("xml:base"), null, baseURI);

        if (title != null)
            root.setAttribute("tooltiptext", title);

        el = this.firstChildElement(el);

        try {
            while (el) {
                switch (el.namespaceURI) {
                case "http://www.barml.b7.pl/2003/barml":
                    switch (el.localName) {
                    case "link":
                        root.appendChild(this.buildLink(el, null, title, baseURI));
                        break;
                    case "group":
                        root.appendChild(this.buildGroup(el, null, title, baseURI));
                        break;
                    case "separator":
                        root.appendChild(this.buildSeparator(el, null, title, baseURI));
                        break;
                    default:
                        return false;
                    }
                    break;
                }
                el = this.nextSiblingElement(el);
            }
        } catch (ex) {
            throw ex;
            return false;
        }

        return true;
    },

    buildLink: function(el, parent, title, baseURI)
    {
        var image = this.firstChildElement(el);
        var target, href, tag;

        if (image && (image.localName != "image" ||
                this.nextSiblingElement(image)) ||
                !el.hasAttribute("href"))
            throw "Invalid Tag1";

        if (el.hasAttribute("xml:base"))
            baseURI = ios.newURI(el.getAttribute("xml:base"), null, baseURI);

        image = image && image.hasAttribute("src") && 
            baseURI.resolve(image.getAttribute("src"));

        if (image && !barMLHandler.securityCheck(image))
            throw "Invalid TagX";

        target = el.getAttribute("target") || "_self";
        href = baseURI.resolve(el.getAttribute("href"));

        if (parent)
            tag = this.createElement("menuitem", this.xulns,
                "class", "menuitem-iconic bookmark-item",
                "label", this.getText(el),
                "image", image,
                "tooltiptext", el.getAttribute("title") || title);
        else
            tag = this.createElement("toolbarbutton", this.xulns,
                "class", "bookmark-item",
                "label", this.getText(el),
                "image", image,
                "tooltiptext", el.getAttribute("title") || title);

        if (parent)
            tag.setAttribute("type", "menubutton");

        if (this.baseURI.spec != href)
            tag.setAttribute("oncommand", "barMLHandler.openLink('"+href+"', '"+target+"')");
        else
            tag.setAttribute("disabled", "true");

        return tag;
    },

    buildGroup: function(group, parent, title, baseURI)
    {
        var root, menupopup, image, el;

        if (parent)
            root = this.createElement("menu", this.xulns,
                "class", "menu-iconic bookmark-item",
                "container", "true",
                "type", "menu");
        else
            root = this.createElement("toolbarbutton", this.xulns,
                "class", "bookmark-item",
                "container", "true",
                "type", "menu");

        title = group.getAttribute("title") || title;

        menupopup = this.createElement("menupopup", this.xulns);
        root.appendChild(menupopup);

        if (title)
            menupopup.setAttribute("tooltiptext", title);

        if (group.hasAttribute("xml:base")) {
            baseURI = ios.newURI(group.getAttribute("xml:base"), null, baseURI);
        }

        el = this.firstChildElement(group);
        while (el) {
            switch (el.namespaceURI) {
            case "http://www.barml.b7.pl/2003/barml":
                switch (el.localName) {
                case "link":
                    menupopup.appendChild(this.buildLink(el, group, title, baseURI));
                    break;
                case "group":
                    menupopup.appendChild(this.buildGroup(el, group, title, baseURI));
                    break;
                case "separator":
                    menupopup.appendChild(this.buildSeparator(el, group, title, baseURI));
                    break;
                case "name":
                    image = this.firstChildElement(el);

                    if (image && image.localName != "image" &&
                            this.nextSiblingElement(image))
                        throw "Invalid Tag2";

                    image = image && image.hasAttribute("src") && 
                        baseURI.resolve(image.getAttribute("src"));

                    if (image && !barMLHandler.securityCheck(image))
                        throw "Invalid Tag";

                    if (image)
                        root.setAttribute("image", image);

                    root.setAttribute("label", this.getText(el));
                    if (el.hasAttribute("title"))
                        root.setAttribute("tooltiptext", el.getAttribute("title"))
                    break;
                default:
                    throw "Invalid Tag3";
                }
                break;
            }
            el = this.nextSiblingElement(el);
        }
        if (!root.hasAttribute("tooltiptext") && title)
            root.setAttribute("tooltiptext", title);

        return root;
    },

    buildSeparator: function(el, parent, title, baseURI)
    {
        var tag = this.firstChildElement(el);

        if (tag || el.attributes.length)
            throw "Invalid Tag4";

        if (parent)
            return this.createElement("menuseparator", this.xulns,
                "tooltiptext", el.getAttribute("title") || title);
        else
            return this.createElement("toolbarseparator", this.xulns,
                "tooltiptext", el.getAttribute("title") || title);
    },

    getText: function(el)
    {
        var i, ret;

        if (el.nodeType == el.ELEMENT_NODE) {
            ret = "";
            for (i = 0; i < el.childNodes.length; i++)
                ret += this.getText(el.childNodes[i]);
            return ret;
        } else if (el.nodeType == el.TEXT_NODE)
            return el.nodeValue;
    },

    parentElement: function(el)
    {
        while (el && ! (el.parentNode instanceof Components.interfaces.nsIDOMElement))
            el = el.parentNode;
        return el;
    },

    createElement: function(name, ns)
    {
        var tag = document.createElementNS(ns, name);
        var i;

        for (i = 2; i < arguments.length; i+=2)
            if (arguments[i+1])
                tag.setAttribute(arguments[i], arguments[i+1]);
        return tag;
    },

    firstChildElement: function(el)
    {
        if (!el)
            return null;

        el = el.firstChild;

        while (el && !(el instanceof Components.interfaces.nsIDOMElement))
            el = el.nextSibling;

        return el;
    },

    nextSiblingElement: function(el)
    {
        if (!el)
            return null;

        el = el.nextSibling;

        while (el && !(el instanceof Components.interfaces.nsIDOMElement))
            el = el.nextSibling;

        return el;
    }
}

var barMLHandler = new BarMLHandler();